@@ -1,5 +1,15 @@
-# Verilog - Verilog Perl Interface
-# See copyright, etc in below POD section.
+#$Id: Preproc.pm 39293 2007-05-23 19:39:17Z wsnyder $
+######################################################################
+#
+# Copyright 2001-2007 by Wilson Snyder.  This program is free software;
+# you can redistribute it and/or modify it under the terms of either the GNU
+# General Public License or the Perl Artistic License.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
 ######################################################################
 
 package Verilog::Preproc;
@@ -7,11 +17,12 @@ use Carp;
 use Verilog::Getopt;
 
 require DynaLoader;
-use base qw(DynaLoader);
+@ISA = qw(DynaLoader);
+
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '2.468';
+$VERSION = '3.000';
 
 ######################################################################
 #### Configuration Section
@@ -19,9 +30,8 @@ $VERSION = '2.468';
 bootstrap Verilog::Preproc;
 
 #In Preproc.xs:
-# sub _new (class, keepcmt, linedir, pedantic, synthesis)
+# sub _new (class, keepcmt, linedir, pedantic)
 # sub _open (class)
-# sub getall (class)
 # sub getline (class)
 # sub eof (class)
 # sub filename (class)
@@ -36,11 +46,8 @@ sub new {
     my $self = {keep_comments=>1,
 		keep_whitespace=>1,
 		line_directives=>1,
-		ieee_predefined=>1,
 		pedantic=>0,
-		synthesis=>0,
 		options=>Verilog::Getopt->new(),	# If the user didn't give one, still work!
-		parent => undef,
 		#include_open_nonfatal=>0,
 		@_};
     bless $self, $class;
@@ -52,45 +59,11 @@ sub new {
 		$self->{keep_whitespace},
 		$self->{line_directives},
 		$self->{pedantic},
-		$self->{synthesis},
 		);
-    if ($self->{synthesis}) {
-	# Fourth argument 1 for cmdline - no `undefineall effect
-	$self->define('SYNTHESIS',1,undef,1);
-    }
-    if ($self->{ieee_predefined}) {
-	$self->define('SV_COV_START', 0,undef,1);
-	$self->define('SV_COV_STOP', 1,undef,1);
-	$self->define('SV_COV_RESET', 2,undef,1);
-	$self->define('SV_COV_CHECK', 3,undef,1);
-	$self->define('SV_COV_MODULE', 10,undef,1);
-	$self->define('SV_COV_HIER', 11,undef,1);
-	$self->define('SV_COV_ASSERTION', 20,undef,1);
-	$self->define('SV_COV_FSM_STATE', 21,undef,1);
-	$self->define('SV_COV_STATEMENT', 22,undef,1);
-	$self->define('SV_COV_TOGGLE', 23,undef,1);
-	$self->define('SV_COV_OVERFLOW', -2,undef,1);
-	$self->define('SV_COV_ERROR', -1,undef,1);
-	$self->define('SV_COV_NOCOV', 0,undef,1);
-	$self->define('SV_COV_OK', 1,undef,1);
-	$self->define('SV_COV_PARTIAL', 2,undef,1);
-    }
     #use Data::Dumper; print Dumper($self);
     return $self;
 }
 
-sub DESTROY {
-    my $self = shift;
-    $self->_DESTROY;
-}
-
-sub STORABLE_freeze {
-    my ($self, $cloning) = @_;
-    # Prevent crash on Storable::store then retrieve
-    delete $self->{_cthis};
-    return;
-}
-
 sub open {
     my $self = shift;
     my %params = (
@@ -126,11 +99,6 @@ sub debug {
     $self->_debug($level);
 }
 
-sub parent {
-    my $self = shift;
-    return $self->{parent};
-}
-
 ######################################################################
 #### Utilities
 
@@ -164,10 +132,6 @@ sub error {
 
 sub comment {}
 
-sub def_substitute {
-    my ($self, $out) = @_;
-    return $out;
-}
 sub include {
     my ($self,$filename)=@_;
     print "INCLUDE $filename\n" if $self->{debug};
@@ -176,17 +140,13 @@ sub include {
 		);
 }
 
-# Note rather than overriding these, a derived Verilog::Getopt class can
+# Note rather then overriding these, a derived Verilog::Getopt class can
 # accomplish the same thing.
 
 sub undef {
     my $self = shift;
     $self->{options}->undef(@_);
 }
-sub undefineall {
-    my $self = shift;
-    $self->{options}->undefineall(@_);
-}
 sub define {
     my $self = shift;
     #print "DEFINE @_\n";
@@ -230,7 +190,7 @@ Verilog::Preproc - Preprocess Verilog files
 =head1 EXAMPLE
 
     # This is a complete verilog pre-parser!
-    # For a command line version, see vppreproc
+    # For a command line version, see vppp
     use Verilog::Getopt;
     use Verilog::Preproc;
 
@@ -246,12 +206,12 @@ Verilog::Preproc - Preprocess Verilog files
 =head1 DESCRIPTION
 
 Verilog::Preproc reads Verilog files, and preprocesses them according to
-the SystemVerilog 2009 (1800-2009) specification.  Programs can be easily
-converted from reading a IO::File into reading preprocessed output from
+the SystemVerilog 2005 specification.  Programs can be easily converted
+from reading a IO::File into reading preprocessed output from
 Verilog::Preproc.
 
-See the "Which Package" section of L<Verilog-Perl> if you are unsure which
-parsing package to use for a new application.
+See the "Which Package" section of L<Verilog::Language> if you are unsure
+which parsing package to use for a new application.
 
 =head1 MEMBER FUNCTIONS
 
@@ -266,12 +226,6 @@ Returns true at the end of the file.
 Returns the filename of the most recently returned getline().  May not match
 the filename passed on the command line, as `line directives are honored.
 
-=item $self->getall()
-
-Return the entire translated text up to the final EOF, similar to calling
-join('',$self->getline) but significantly faster.  With optional argument,
-returns approximately that number of characters.  Returns undef at EOF.
-
 =item $self->getline()
 
 Return the next line of text.  Returns undef at EOF.  (Just like
@@ -283,11 +237,6 @@ Returns the line number of the last getline().  Note that the line number
 may change several times between getline(), for example when traversing
 multiple include files.
 
-=item $self->parent()
-
-Returns a reference to the Verilog::Netlist::File which created this
-object, if any.
-
 =item $self->new(I<parameters>)
 
 Creates a new preprocessor.  See the PARAMETERS section for the options
@@ -295,10 +244,9 @@ that may be passed to new.
 
 =item $self->open(filename=>I<filename>)
 
-Opens the specified file.  If filename ends in .gz, decompress while
-reading.  If called before a file is completely parsed, the new file will
-be parsed completely before returning to the previously open file.  (As if
-it was an include file.)
+Opens the specified file.  If called before a file is completely parsed,
+the new file will be parsed completely before returning to the previously
+open file.  (As if it was an include file.)
 
 Open may also be called without named parameters, in which case the only
 argument is the filename.
@@ -317,11 +265,6 @@ The following named parameters may be passed to the new constructor.
 
 =over 4
 
-=item ieee_predefines=>0
-
-With ieee_predefines false, disable defining SV_COV_START and other IEEE
-mandated definitions.
-
 =item include_open_nonfatal=>1
 
 With include_open_nonfatal set to one, ignore any include files that do
@@ -333,8 +276,8 @@ With keep_comments set to zero, strip all comments.  When set to one (the
 default), insert comments in output streams.  When set to 'sub', call the
 comment() function so that meta-comments can be processed outside of the
 output stream.  Note that some programs use meta-comments to embed useful
-information (synthesis and lint), so strip with caution if feeding to tools
-other than your own.  Defaults to 1.
+information (synthesis and lint), so use this with caution if feeding to
+tools other then your own.  Defaults to 1.
 
 =item keep_whitespace=>0
 
@@ -355,19 +298,9 @@ classes may be used, as long as their interface matches that of Getopt.
 
 =item pedantic=>1
 
-With pedantic set, rigorously obey the Verilog pedantic.  This used to
-disable the `__FILE__ and `__LINE__ features but no longer does as they
-were added to the 1800-2009 standard.  It remains to disable `error and may
-disable other future features that are not specified in the language
-standard. Defaults false.
-
-=item synthesis=>1
-
-With synthesis set, define SYNTHESIS, and ignore text between "ambit",
-"pragma", "synopsys" or "synthesis" translate_off and translate_on meta
-comments.  Note using metacomments is discouraged as they have led to
-silicon bugs (versus ifdef SYNTHESIS); see
-L<http://www.veripool.org/papers/TenIPEdits_SNUGBos07_paper.pdf>.
+With pedantic set, rigorously obey the Verilog pedantic.  This disables
+the `__FILE__ and `__LINE__ features, and may disable other features that
+are not specified in the language pedantic. Defaults false.
 
 =back
 
@@ -387,10 +320,6 @@ do nothing.
 
 Called with each `undef.  Defaults to use options object.
 
-=item $self->undefineall()
-
-Called with each `undefineall.  Defaults to use options object.
-
 =item $self->define(I<defname>, I<value>, I<params>)
 
 Called with each `define.  Defaults to use options object.
@@ -401,14 +330,6 @@ Called to determine if the define exists.  Return true if the define
 exists, or argument list with leading parenthesis if the define has
 arguments.  Defaults to use options object.
 
-=item $self->def_substitute(I<string>)
-
-Called to determine what string to insert for a define substitution.
-Called with the value of the define after parameters have been expanded
-computed per the SystemVerilog spec.  Generally this function would just
-return the same string as it is passed, but this can be overridden to allow
-customized preprocessing.
-
 =item $self->def_value(I<defname>)
 
 Called to return value to substitute for specified define.  Defaults to use
@@ -428,28 +349,21 @@ after resolving the filename with the options parameter.
 
 =head1 COMPLIANCE
 
-The preprocessor supports the constructs defined in the SystemVerilog 2017
-standard (IEEE 1800-2017), which is a superset of Verilog 1995 (IEEE
-1364-1995), Verilog 2001 (IEEE 1364-2001), Verilog 2005 (IEEE 1364-2005)
-SystemVerilog 2005 (IEEE 1800-2005), SystemVerilog 2009 (IEEE
-1800-2009), and SystemVerilog 2012 (IEEE 1800-2012).
+The preprocessor supports the constructs defined in the Verilog 2005 and
+SystemVerilog 2005 standards.
 
-Verilog::Preproc adds the `error macro (unless the pedantic parameter is
-set.):
+Verilog::Preproc adds the following features (unless the pedantic parameter
+is set.):
 
 =over 4
 
 =item `__FILE__
 
-The __FILE__ define expands to the current filename as a string, like C++'s
-__FILE__.  This was incorporated into to the 1800-2009 standard (but
-supported by Verilog-Perl since 2004!)
+`__FILE__ will be replaced by the current filename. (Like C++ __FILE__.)
 
 =item `__LINE__
 
-The __LINE__ define expands to the current filename as a string, like C++'s
-__LINE__.  This was incorporated into to the 1800-2009 standard (but
-supported by Verilog-Perl since 2004!)
+`__LINE__  will be replaced by the current line number. (Like C++ __LINE__.)
 
 =item `error I<"string">
 
@@ -461,13 +375,13 @@ These are useful for error macros, similar to assert() in C++.
 
 =head1 DISTRIBUTION
 
-Verilog-Perl is part of the L<http://www.veripool.org/> free Verilog EDA
+Verilog-Perl is part of the L<http://www.veripool.com/> free Verilog EDA
 software tool suite.  The latest version is available from CPAN and from
-L<http://www.veripool.org/verilog-perl>.
+L<http://www.veripool.com/verilog-perl.html>.
 
-Copyright 2000-2019 by Wilson Snyder.  This package is free software; you
+Copyright 2000-2007 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
-Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
+Lesser General Public License or the Perl Artistic License.
 
 =head1 AUTHORS
 
@@ -478,7 +392,7 @@ Wilson Snyder <wsnyder@wsnyder.org>
 L<Verilog-Perl>,
 L<Verilog::Language>, L<Verilog::Getopt>
 
-L<IO::File>
+L<IO::File> 
 
 This package is layered on a C++ interface which may be found in the kit.
 
